import requests
import uuid
import os
import textwrap
import json
import subprocess
import urllib
from typing import List, Callable, Tuple
from vessel.pipeline import Step, Payload
from vessel.logging import logger

class DeployAgentStep(Step):
  """
  Deploy Agent
  """
  def __init__(self):
    super().__init__()

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    start_fn("Deploying agent")
    yaml = os.path.expanduser(f"~/.daas/{payload.token}/agent.yaml")
    bashCommand = f"kubectl -n daas apply -f {yaml}"
    process = subprocess.Popen(bashCommand.split(), stdout=subprocess.PIPE)
    output, error = process.communicate()
    if error:
      raise Exception(error)
    for out in output.decode('utf-8').split("\n"):
      end_fn(out)
    return payload

class DeploySentinelStep(Step):
  """
  Generate kube deployments for agent & sentinel
  """

  def __init__(self):
    super().__init__()

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    start_fn("Deploying sentinel")
    yaml = os.path.expanduser(f"~/.daas/{payload.token}/sentinel.yaml")
    bashCommand = f"kubectl -n daas apply -f {yaml}"
    process = subprocess.Popen(bashCommand.split(), stdout=subprocess.PIPE)
    output, error = process.communicate()
    if error:
      raise Exception(error)
    for out in output.decode('utf-8').split("\n"):
      end_fn(out)
    return payload

class GenerateYamlStep(Step):
  """
  Generate kube deployments for agent & sentinel
  """

  def __init__(self):
    super().__init__()

  def _get_keys(self, payload:Payload) -> Tuple[str]:
    if payload.rsa:
      name = urllib.parse.quote(payload.cluster['result']['name'].lower().replace(" ", "-"))
      return (name, payload.rsa, payload.vault_user, payload.vault_pwd, payload.distribution)
    else:
      with open(os.path.expanduser(f"~/.daas/{payload.token}/registration.json"), 'r') as f:
        registration = json.load(f)
      with open(os.path.expanduser(f"~/.daas/vault.json"), 'r') as f:
        vault = json.load(f)
      
      name = urllib.parse.quote(registration['cluster']['result']['name'].lower().replace(" ", "-"))
      return (name, registration['rsa'], vault['public']['user'], vault['public']['pwd'], registration['distribution'])



  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    start_fn("Loading keys")
    path = os.path.expanduser(f"~/.daas/{payload.token}")
    
    (name, rsa, vault_user, vault_pwd, distribution) = self._get_keys(payload)

    start_fn("generating sentinel yaml")
    sentinel_yaml = self.sentinel_yaml(name=name, token=payload.token, vault_password=vault_pwd, distribution=distribution, tag='latest')
    
    with open(f"{path}/sentinel.yaml", 'w') as f:
      f.write(sentinel_yaml)

    start_fn("generating agent yaml")
    agent_yaml = self.agent_yaml(name=name, token=payload.token, rsa=rsa.replace("\n", "\\n"), tag='latest')
   
    with open(f"{path}/agent.yaml", 'w') as f:
      f.write(agent_yaml)
    
    payload.agent_yaml = agent_yaml
    payload.sentinel_yaml = sentinel_yaml
    end_fn("OK")

    return payload


  def sentinel_yaml(self, **kwargs):
    """
    Output the deployemnt for the Sentinel
    """
    return textwrap.dedent(
        """\
        apiVersion: v1
        kind: List
        metadata:
          resourceVersion: ""
          selfLink: ""
        items:
        - apiVersion: v1
          kind: Secret
          metadata:
            name: vault-secret
          type: Opaque
          stringData:
            HOST: "http://vault.vault:8200"
            USER: "sourcesense"
            PWD: "{vault_password}"
        - apiVersion: v1
          kind: ConfigMap
          metadata:
            name: sentinel-tasks-cm
          data:
            tasks.json: |
              {{
                "tasks": [
                  {{
                    "name": "kubelinter",
                    "module": "sentinel.tasks.kubelinter.KubeLinter",
                    "enabled": true,
                    "scheduled": 3600
                  }},
                  {{
                    "name": "kubecounter",
                    "module": "sentinel.tasks.kubecounter.KubeCounter",
                    "enabled": true,
                    "scheduled": 3600
                  }},
                  {{
                    "name": "anchore",
                    "module": "sentinel.tasks.anchore.Anchore",
                    "enabled": true,
                    "scheduled": 43200
                  }},
                  {{
                    "name": "events",
                    "module": "sentinel.tasks.events.Events",
                    "enabled": true
                  }}
                ]
              }}
        - apiVersion: apps/v1
          kind: Deployment
          metadata:
            name: {name}-sentinel
          spec:
            selector:
              matchLabels:
                app: {name}-sentinel
            replicas: 1
            template:
              metadata:
                name: {name}-sentinel
                labels:
                  app: {name}-sentinel
              spec:
                volumes:
                  - name: tasks-volume
                    configMap:
                      name: sentinel-tasks-cm
                containers:
                - name: {name}-sentinel
                  image: docker-registry.oc.corp.sourcesense.com/daas/workstation-sentinel:{tag}
                  volumeMounts:
                    - name: tasks-volume
                      mountPath: /etc/custom
                  env:
                  - name: CLUSTER_TOKEN
                    value: {token}
                  - name: TASKS_JSON
                    value: /etc/custom/tasks.json 
                  - name: KUBE_DISTRIBUTION
                    value: {distribution}
                  - name: VAULT_HOST
                    valueFrom:
                      secretKeyRef:
                        name: vault-secret
                        key: HOST
                  - name: VAULT_USER
                    valueFrom:
                      secretKeyRef:
                        name: vault-secret
                        key: USER
                  - name: VAULT_PWD
                    valueFrom:
                      secretKeyRef:
                        name: vault-secret
                        key: PWD
                  - name: ANCHORE_URL
                    value: http://anchore-anchore-engine-api.anchore.svc:8228/anchore
                  - name: ANCHORE_USERNAME
                    value: admin
                  - name: ANCHORE_PWD
                    valueFrom:
                      secretKeyRef:
                        name: anchore-password
                        key: ANCHORE_PWD
                  - name: BEAT_OUTPUT
                    value: filebeat.daas.svc:9000
                imagePullSecrets:
                - name: sourcesense-registry
        - apiVersion: v1
          kind: Service
          metadata:
            labels:
              app: {name}-sentinel
            name: svc-{name}-sentinel
          spec:
            ports:
            - name: 8089-tcp
              port: 8089
              protocol: TCP
              targetPort: 8089
            selector:
              app: {name}-sentinel
            sessionAffinity: None
            type: ClusterIP
        - apiVersion: extensions/v1beta1
          kind: Ingress
          metadata:
            name: {name}-sentinel-ingress
          spec:
            rules:
            - host: {name}-sentinel.local
              http:
                paths:
                - backend:
                    serviceName: svc-{name}-sentinel
                    servicePort: 8089
                  path: /
        """
        ).format(**kwargs)


  def agent_yaml(self, **kwargs):
    """
    Output the deployemnt and secret for the agent
    """
    return textwrap.dedent(
        """\
        apiVersion: v1
        kind: List
        metadata:
          resourceVersion: ""
          selfLink: ""
        items:
        - apiVersion: v1
          kind: Secret
          metadata:
            name: {name}-secrets
          type: Opaque
          stringData:
            PRIVATE_KEY: "{rsa}"
        - apiVersion: apps/v1
          kind: Deployment
          metadata:
            name: {name}-agent
          spec:
            selector:
              matchLabels:
                app: {name}-agent
            replicas: 1
            template:
              metadata:
                labels:
                  app: {name}-agent
              spec:
                containers:
                - name: {name}-agent
                  image: docker-registry.oc.corp.sourcesense.com/daas/workstation-agent:{tag}
                  env:
                  - name: CLUSTER_TOKEN
                    value: {token}
                  - name: VAULT_HOST
                    valueFrom:
                      secretKeyRef:
                        name: vault-secret
                        key: HOST
                  - name: VAULT_USER
                    valueFrom:
                      secretKeyRef:
                        name: vault-secret
                        key: USER
                  - name: VAULT_PWD
                    valueFrom:
                      secretKeyRef:
                        name: vault-secret
                        key: PWD
                  - name: PRIVATE_KEY
                    valueFrom:
                      secretKeyRef:
                        name: {name}-secrets
                        key: PRIVATE_KEY
                imagePullSecrets:
                - name: sourcesense-registry


        """
        ).format(**kwargs)


import requests
import uuid
import os
from typing import List, Callable
from vessel.pipeline import Step, Payload
from vessel.logging import logger

class RegisterStep(Step):
  """
  Register to Vessel api the cluster
  """

  def __init__(self, host):
    super().__init__()
    self.host = host

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    start_fn("Registering to api")
    
    rpc_payload = {
        "method": "agent.register",
        "params": {
          "t": payload.token,
          "iv": payload.vault_iv,
          "ct": payload.vault_ct,
          "pub": payload.rsa_pub
        },
        "jsonrpc": "2.0",
        "id": str(uuid.uuid4()),
    }
    response = requests.post(f"{self.host}", json=rpc_payload)
    if response.status_code >= 400 or response.json().get("error", False):
      raise Exception(response.text)

    payload.cluster = response.json()
    
    # save registration request
    start_fn("Saving registration")
    path = os.path.expanduser(f"~/.daas/{payload.token}")
    if not os.path.exists(path):
      os.mkdir( path )
    with open(f"{path}/registration.json", 'w') as f:
      f.write(payload.registration_json())

    end_fn(f"OK [ {path} ]")
    return payload



from pyfiglet import Figlet
import click
import time
import os
import sys
import json
import urllib
import urllib3
from urllib.parse import urljoin

from vessel.steps import VaultInitStep, VaultSaveSecretsStep, GenerateKeysStep, \
  RegisterStep, VaultUnsealStep, GenerateYamlStep, DeployAgentStep, DeploySentinelStep
from vessel.pipeline import Pipeline, Payload
from vessel.logging import logger
from vessel.version import VERSION
import kubernetes
from kubernetes.client.rest import ApiException

LATEST_AGENT="1.0"
LATEST_SENTINEL="1.1.1"

def start(msg):
  click.echo(f"\n> {msg}...")

def end(msg):
  click.echo(f"[*] {msg}")

def prompt(msg, tag, **kwargs):
  return click.prompt(msg, type=str, **kwargs)

@click.group( invoke_without_command=True)
@click.option('--debug', is_flag=True, default=False, help="output debug log [False]")
@click.option('--version', is_flag=True, default=False,  help="Show version and exits")
@click.pass_context
def main(ctx, debug, version):
  """
  Vessel cli tool 
  """
  if version:
    print(f"Vessel cli tool: {VERSION}")
    sys.exit()

  if ctx.invoked_subcommand is None:
    print(ctx.get_help())
    sys.exit()

  click.clear()
  f = Figlet(font='standard')
  click.echo(f.renderText('Vessel cli tool'))
  ctx.ensure_object(dict)
  ctx.obj['DEBUG'] = debug

  if debug:
    logger.setLevel("DEBUG")
  
  # Init directories
  if not os.path.exists(os.path.expanduser("~/.daas")):
      os.mkdir( os.path.expanduser("~/.daas"))

@main.command()
@click.pass_context
@click.option('--vault', default='http://vault.local', help='Vault endpoint [http://vault.local]')
def init(ctx, vault):
  """
  Init vault
  """
  payload = Payload()
  pipeline = Pipeline(start_fn=start, end_fn=end, prompt_fn=prompt)
  pipeline.add(VaultInitStep(vault))
  try:
    # Run pipeline
    payload = pipeline.run(payload)    
  except Exception as e:
    logger.error(e)
    if ctx.obj['DEBUG']:
      raise e

@main.command()
@click.pass_context
@click.option('--vault', default='http://vault.local', help='Vault endpoint [http://vault.local]')
def unseal(ctx, vault):
  """
  Unseal vault
  """
  payload = Payload()
  pipeline = Pipeline(start_fn=start, end_fn=end, prompt_fn=prompt)
  pipeline.add(VaultUnsealStep(vault))
  try:
    # Run pipeline
    payload = pipeline.run(payload)    
  except Exception as e:
    logger.error(e)
    if ctx.obj['DEBUG']:
      raise e

@main.command()
@click.pass_context
@click.argument('token')
@click.option('--cluster-host', required=True, help="Hostname of the cluster to control")
@click.option('--cluster-ro', required=True, help="Cluster read-only service-account token")
@click.option('--cluster-rw', required=True, help="Cluster read-write service-account token")
@click.option('--vault', default='http://vault.local', help='Vault endpoint [http://vault.local]')
@click.option('--openshift', is_flag=True, default=False, help="Cluster is an Openshift distribution [False]")
@click.option('--init', is_flag=True, default=False, help="Initialize Vault [False]")
@click.option('--deploy', is_flag=True, default=False, help="Deploy agent and sentinel container automatically [False]")
@click.option('--vessel-api', default="http://cloud-api.oc.corp.sourcesense.com/rpc", help="Vessel API RPC endpoint [http://cloud-api.oc.corp.sourcesense.com/rpc]")
def register(ctx, token, cluster_host, cluster_ro, cluster_rw, vault, openshift, init, deploy, vessel_api):
  """
  Register workstaion to Vessel with the given TOKEN
  """
  payload = Payload(token)
  pipeline = Pipeline(start_fn=start, end_fn=end, prompt_fn=prompt)
  if init:
    pipeline.add(VaultInitStep(vault))
  pipeline.add(VaultSaveSecretsStep(vault, cluster_host, cluster_ro, cluster_rw, openshift))
  pipeline.add(GenerateKeysStep())
  pipeline.add(RegisterStep(vessel_api))
  pipeline.add(GenerateYamlStep())
  if deploy:
    pipeline.add(DeployAgentStep())
    pipeline.add(DeploySentinelStep())

  try:
    # Run pipeline
    payload = pipeline.run(payload)    
  except Exception as e:
    logger.error(e)
    if ctx.obj['DEBUG']:
      raise e
  

@main.command()
@click.pass_context
@click.argument('token')
@click.option('--dry', is_flag=True, default=False, help="Regenerates yaml files [False]")
def deploy(ctx, token, dry):
  """
  Deploy agent and sentinel for given TOKEN
  """
  click.echo('Debug is %s' % (ctx.obj['DEBUG'] and 'on' or 'off'))
  payload = Payload(token)
  pipeline = Pipeline(start_fn=start, end_fn=end, prompt_fn=prompt)
  pipeline.add(GenerateYamlStep())

  if not dry:
    pipeline.add(DeployAgentStep())
    pipeline.add(DeploySentinelStep())

  try:
    # Run pipeline
    payload = pipeline.run(payload)    
  except Exception as e:
    logger.error(e)
    if ctx.obj['DEBUG']:
      raise e

def patch_image(component, deployment, namespace, image, kube_apps):
  try:
      body = {
        "spec": {
          "template": {
            "spec": {
              "containers": [
                {
                  "name": deployment,
                  "image": image
                  }
              ]
            }
          }
        }
      }
      api_response = kube_apps.patch_namespaced_deployment(deployment, namespace, body)
  except ApiException as e:
      print("Exception when calling AppsV1Api->patch_namespaced_deployment: %s\n" % e)


@main.command()
@click.pass_context
@click.argument('token')
@click.option('--agent-tag', default=LATEST_AGENT, help="Set image tag for agent deployment")
@click.option('--sentinel-tag', default=LATEST_SENTINEL, help="Set image tag for sentinel deployment")
@click.option('--vault', default='http://vault.local', help='Vault endpoint [http://vault.local]')
def update(ctx, token, vault, sentinel_tag, agent_tag):
  """
  Updates agent and sentinel deployments
  """
  urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
  payload = Payload(token)

  with open('/home/rke/.vessel/devops.json') as f:
    devops = json.load(f)

  with open(os.path.expanduser(f"~/.daas/{payload.token}/registration.json"), 'r') as f:
    registration = json.load(f)

  namespace = devops['vessel_namespace']
  img_base = devops['images_base_url']
  k8s_token = devops['devops_sa_token']
  cluster_name = urllib.parse.quote(registration['cluster']['result']['name'].lower().replace(" ", "-"))

  # Init kube client
  click.echo('Creating kubernetes client configuration...')
  configuration = kubernetes.client.Configuration()
  configuration.host = devops['kubernetes_url']
  configuration.verify_ssl = False
  configuration.api_key = {"authorization": "Bearer " + k8s_token}
  api_client = kubernetes.client.ApiClient(configuration)
  apps = kubernetes.client.AppsV1Api(api_client)

  agent_deployment = cluster_name + '-agent'
  click.echo('Agent deployment name is %s' % agent_deployment)

  sentinel_deployment = cluster_name + '-sentinel'
  click.echo('Sentinel deployment name is %s' % sentinel_deployment)

  running_agent = apps.read_namespaced_deployment(agent_deployment, namespace).spec.template.spec.containers[0].image
  click.echo('At moment, you are running this tag image for Vessel Agent => ' + running_agent.split(":")[-1] )
  running_sentinel = apps.read_namespaced_deployment(sentinel_deployment, namespace).spec.template.spec.containers[0].image
  click.echo('At moment, you are running this tag image for Vessel Sentinel => ' + running_sentinel.split(":")[-1] )

  if running_agent.split(":")[-1] != agent_tag:
    click.echo('Updating Agent to tag ' + agent_tag)
    image = urljoin(img_base, 'workstation-agent') + ':' + agent_tag
    patch_image('Agent', agent_deployment, namespace, image, apps)
  else:
    click.echo('Agent is already updated')

  if running_sentinel.split(":")[-1] != sentinel_tag: 
    click.echo('Updating Sentinel to tag ' + sentinel_tag)
    image = urljoin(img_base, 'workstation-sentinel') + ':' + sentinel_tag
    patch_image('Sentinel', sentinel_deployment, namespace, image, apps)
  else:
    click.echo('Sentinel is already updated')



import requests
import base64
import os
import sys
import json
import secrets
import string
import hvac

from typing import List, Callable
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.backends import default_backend

from vessel.pipeline import Step, Payload
from vessel.logging import logger



class VaultBaseStep(Step):
  """
  Initialization of Vault
  """
  def __init__(self, host):
    super().__init__()

    self.vault = hvac.Client(url=host)
    self.salt = "6tS/JDZ/ZEMJisfgbvPQaA==" # b'\xea\xd4\xbf$6\x7fdC\t\x8a\xc7\xe0n\xf3\xd0h'
    self.vault_json_path = os.path.expanduser(f"~/.daas/vault.json")
    self.kdf = kdf = PBKDF2HMAC(
      algorithm=hashes.SHA256(),
      length=32,
      salt=base64.b64decode(self.salt),
      iterations=100000,
      backend=default_backend()
    )

    with open('/home/rke/.vessel/devops.json') as f:
      data = json.load(f)
    self.devops_sa = data['devops_sa']
    self.devops_sa_token = data['devops_sa_token']

  def encrypt(self, vault, pwd):
    """
    it encrypts the token using a generated key:
      - PBKDF2HMAC SHA256 SALT 32byte (256bit) 100000 iteration
      - AES OFB no padding
    """
    
    # generate Key
    key = self.kdf.derive(pwd.encode())
    iv = os.urandom(16)

    cipher = Cipher(
      algorithms.AES(key), 
      modes.OFB(iv), 
      backend=default_backend()
    )
    encryptor = cipher.encryptor()

    ct = encryptor.update(f"daas::{vault}".encode()) + encryptor.finalize()
    return (base64.b64encode(iv).decode(), base64.b64encode(ct).decode())

  def decrypt(self, ct, iv, pwd):
    key = self.kdf.derive(pwd.encode())
    cipher = Cipher(
      algorithms.AES(key), 
      modes.OFB(base64.b64decode(iv)), 
      backend=default_backend()
    )
    decryptor = cipher.decryptor()
    clear_text:str =  decryptor.update(base64.b64decode(ct)).decode()
    logger.debug(clear_text)
    if not clear_text.startswith("daas::"):
      raise Exception("Wrong password!")
    return json.loads(clear_text.replace("daas::", ""))

  def unseal(self, keys):
    unseal_response = self.vault.sys.submit_unseal_keys(keys=keys)
    if unseal_response["sealed"] == False:
      return True
    return False

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    raise Exception("Must implement")

class VaultInitStep(VaultBaseStep):
  """
  Initialize Vault 
  """
  def __init__(self, host):
    super().__init__(host)

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    
    #
    # Init
    #
    start_fn("Init Vault")
    init_result = self.vault.sys.initialize(secret_shares=6, secret_threshold=3)
    
    logger.debug(init_result)
    end_fn("OK")
    
    keys = init_result

    #
    # Unsealing
    #
    start_fn("Unsealing Vault")
    self.unseal(keys["keys"])
    end_fn("OK")

    # Auth client
    self.vault.token = keys["root_token"]

    #
    # Create User
    #
    start_fn("Creating policy and user")
    alphabet = string.ascii_letters + string.digits
    payload.vault_user = "sourcesense"
    payload.vault_pwd  = ''.join(secrets.choice(alphabet) for i in range(12)) 
    
    policy = '''
        path "daas_private" {
            capabilities = ["deny"]
        }
        path "daas_public/*" {
            capabilities = ["create", "read", "update", "delete", "list"]
        }

        path "vessel/*" {
            capabilities = ["create", "read", "update", "delete", "list"]
        }
    '''
    self.vault.sys.create_or_update_policy(
        name='daas',
        policy=policy,
    )
    self.vault.sys.enable_auth_method(method_type="userpass", path="userpass")
    self.vault.create_userpass(payload.vault_user, payload.vault_pwd, policies=["daas", "default"])
    end_fn(f"Created user {payload.vault_user}")

    #
    # Mount kv storage
    #
    start_fn("Creating kv secrets storage")
    
    for vpath in ["daas_public", "daas_private", "vessel"]:
      self.vault.sys.enable_secrets_engine(backend_type="kv", path=vpath, options={"version": 2})
    
    end_fn("OK")

    start_fn("Needed a password to encrypt vault root tokens")
    pwd = prompt_fn("Enter a password", 5, hide_input=True)
    confirm = None
    while(pwd != confirm):
      confirm = prompt_fn("Reenter password ", 5, hide_input=True)

    start_fn("Encrypting vault token")
    (iv, ct) = self.encrypt(json.dumps(keys), pwd)
    payload.vault_iv = iv
    payload.vault_ct = ct

    # save valut initialization
    with open(self.vault_json_path, 'w') as f:
      f.write(json.dumps({
        "public": {
          "user": payload.vault_user,
          "pwd": payload.vault_pwd
        },
        "iv": payload.vault_iv,
        "ct": payload.vault_ct
      }))
    end_fn("OK")

    return payload


class VaultUnsealStep(VaultBaseStep):
  """
  Unseal Vault Step
  """
  def __init__(self, host):
    super().__init__(host)

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    #
    # Unsealing
    #
    start_fn("Needed a password to decrypt vault root tokens")
    pwd = prompt_fn("Enter password to decrypt", 5, hide_input=True)
    with open(self.vault_json_path, 'r') as f:
      crypted_keys = json.load(f)
      logger.debug(crypted_keys)
      payload.vault_iv = crypted_keys['iv']
      payload.vault_ct = crypted_keys['ct']
      
    keys = self.decrypt(payload.vault_ct, payload.vault_iv, pwd)
    start_fn("Unsealing Vault")
    self.unseal(keys["keys"])
    end_fn("OK")


class VaultSaveSecretsStep(VaultBaseStep):
  def __init__(self, host, cluster_host, cluster_ro_token, cluster_rw_token, openshift):
    super().__init__(host)
    self.cluster_host = cluster_host
    self.cluster_ro_token = cluster_ro_token
    self.cluster_rw_token = cluster_rw_token
    self.openshift = openshift

  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    #
    # Load root token
    # 
    start_fn("Needed a password to decrypt vault root tokens")
    pwd = prompt_fn("Enter password to decrypt", 5, hide_input=True)
    with open(self.vault_json_path, 'r') as f:
      crypted_keys = json.load(f)
      logger.debug(crypted_keys)
      payload.vault_iv = crypted_keys['iv']
      payload.vault_ct = crypted_keys['ct']
      
    keys = self.decrypt(payload.vault_ct, payload.vault_iv, pwd)
    self.vault.token = keys['root_token']

    #
    # Fill payload 
    #
    payload.distribution = "openshift" if self.openshift else "kubernetes"
    
    #
    # Create secret
    #
    start_fn("Storing Secrets")
    
    self.vault.secrets.kv.v2.create_or_update_secret(
        path=payload.token,
        secret={
          "url": self.cluster_host,
          "distribution": payload.distribution,
          "ro-token": self.cluster_ro_token
        },
        mount_point="daas_public"
    )
    self.vault.secrets.kv.v2.create_or_update_secret(
        path='kubernetes',
        secret={
          "devops_sa": self.devops_sa,
          "devops_token": self.devops_sa_token
        },
        mount_point="vessel"
    )
    self.vault.secrets.kv.v2.create_or_update_secret(
        path=payload.token,
        secret={
          "url": self.cluster_host,
          "distribution": payload.distribution,
          "rw-token": self.cluster_rw_token
        },
        mount_point="daas_private"
    )

    end_fn("Stored service-accounts secrets")
    return payload


import requests
from cryptography.hazmat.primitives.asymmetric import rsa 
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from typing import List, Callable
from vessel.pipeline import Step, Payload
from vessel.logging import logger

class GenerateKeysStep(Step):
  """
  RSA key pair generation
  """
  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable, prompt_fn:Callable) -> Payload:
    start_fn("Generating key pair")
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=4096,
        backend=default_backend()
    )
    pub = private_key.public_key().public_bytes(
      serialization.Encoding.PEM, 
      serialization.PublicFormat.SubjectPublicKeyInfo
    )
    pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption()
    )
    payload.rsa = pem.decode()
    payload.rsa_pub = pub.decode()
    end_fn("OK")

    return payload



import logging
logger = logging.getLogger("vessel-cli")
handler = logging.StreamHandler()
formatter = logging.Formatter('[%(levelname)s]: %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
:copyright: (c) 2017 by Lev Lazinskiy
:license: MIT, see LICENSE for more details.
"""

import setuptools
from vessel.version import VERSION

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sourcesense-vessel",
    version=VERSION,
    author="Sourcesense spa",
    author_email="open@sourcesense.com",
    description="Vessel Cli Tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://sourcesense.com",
    license="all rights reserved",
    packages=setuptools.find_packages(),
    scripts=['bin/vessel-cli'],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: Other/Proprietary License",
        "Intended Audience :: System Administrators",
        "Programming Language :: Python :: 3",
    ],
    install_requires=[
          'click',
          'pyfiglet',
          'cryptography',
          'requests',
          'hvac',
          'kubernetes'
      ],
    python_requires='>=3.6',
)


from typing import List, Callable
import json
import click


from vessel.logging import logger

class Payload(object):
  def __init__(self, token:str = None):
    
    self.token:str = token
    self.serviceaccount = []
    self.vault_ct = None
    self.vault_iv = None
    self.vault_user = None
    self.vault_pwd  = None
    self.rsa_pub:str = None
    self.rsa:str = None
    self.cluster = None
    self.distribution = "kubernetes"
    self.agent_yaml = None
    self.sentinel_yaml = None

  def __str__(self):
    return json.dumps(self.__dict__)

  def registration_json(self):
    return json.dumps({
      "token": self.token,
      "rsa_pub": self.rsa_pub,
      "rsa":  self.rsa,
      "distribution": self.distribution,
      "cluster": self.cluster
    })


class Step(object):
  def run(self, payload:Payload, start_fn:Callable, end_fn:Callable) -> Payload:
    raise Exception("Must implement")

class Pipeline(object):
  def __init__(self, start_fn=print, end_fn=print, prompt_fn=input):
    self.steps:List[Step] = []
    self.start_fn = start_fn
    self.end_fn = end_fn
    self.prompt_fn = prompt_fn

  def add(self, step:Step):
    self.steps.append(step)

  def run(self, start_payload:Payload):
    result:Payload = start_payload
    for step in self.steps:
      result = step.run(result, self.start_fn, self.end_fn, self.prompt_fn)
    
    return result
      
      
